package example;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.bindings.keys.ParseException;
import org.eclipse.jface.fieldassist.IContentProposalProvider;
import org.eclipse.jface.fieldassist.SimpleContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.nebula.widgets.nattable.NatTable;
import org.eclipse.nebula.widgets.nattable.config.AbstractRegistryConfiguration;
import org.eclipse.nebula.widgets.nattable.config.DefaultNatTableStyleConfiguration;
import org.eclipse.nebula.widgets.nattable.config.IConfigRegistry;
import org.eclipse.nebula.widgets.nattable.config.IEditableRule;
import org.eclipse.nebula.widgets.nattable.data.IDataProvider;
import org.eclipse.nebula.widgets.nattable.edit.EditConfigAttributes;
import org.eclipse.nebula.widgets.nattable.edit.editor.TextCellEditor;
import org.eclipse.nebula.widgets.nattable.grid.data.DefaultColumnHeaderDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.data.DummyBodyDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.layer.DefaultGridLayer;
import org.eclipse.nebula.widgets.nattable.layer.DataLayer;
import org.eclipse.nebula.widgets.nattable.layer.cell.ColumnOverrideLabelAccumulator;
import org.eclipse.nebula.widgets.nattable.style.DisplayMode;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public class ContentAssistExample {

  static final String COLUMN_ONE_LABEL = "ColumnOne";

  public static void main(String[] args) {
    int shellWidth = 300;
    int shellHeight = 300;
    Display display = Display.getDefault();
    Shell shell = new Shell(display, SWT.SHELL_TRIM);
    shell.setLayout(new FillLayout());
    shell.setSize(shellWidth, shellHeight);
    shell.setText("Content Assist");

    // Create example control
    Control exampleControl = createExampleControl(shell);

    shell.open();

    while (!shell.isDisposed()) {
      if (!display.readAndDispatch()) {
        display.sleep();
      }
    }

    exampleControl.dispose();

    shell.dispose();
    display.dispose();
  }

  public static Control createExampleControl(Composite parent) {
    String[] propertyNames = {"textField"};

    // mapping from property to label, needed for column header labels
    Map<String, String> propertyToLabelMap = new HashMap<>();
    propertyToLabelMap.put("textField", "Text Field");

    IDataProvider bodyDataProvider = new DummyBodyDataProvider(1, 1);

    DefaultGridLayer gridLayer =
        new DefaultGridLayer(bodyDataProvider,
            new DefaultColumnHeaderDataProvider(propertyNames, propertyToLabelMap));

    final DataLayer bodyDataLayer = (DataLayer) gridLayer.getBodyDataLayer();

    final ColumnOverrideLabelAccumulator columnLabelAccumulator =
        new ColumnOverrideLabelAccumulator(bodyDataLayer);
    bodyDataLayer.setConfigLabelAccumulator(columnLabelAccumulator);
    columnLabelAccumulator.registerColumnOverrides(0, COLUMN_ONE_LABEL);

    final NatTable natTable = new NatTable(parent, gridLayer, false);
    natTable.addConfiguration(new DefaultNatTableStyleConfiguration());
    natTable.addConfiguration(new ContentAssistExample.EditorConfiguration());
    natTable.configure();

    return natTable;
  }

  static class EditorConfiguration extends AbstractRegistryConfiguration {

    @Override
    public void configureRegistry(IConfigRegistry configRegistry) {
      TextCellEditor editor = new TextCellEditor();
      char[] autoActivationCharacters = "abcdefghijklmnopqrstuvwxyz ".toCharArray();
      String contentAssistTrigger = "Ctrl+Space";
      String[] proposals = {"one", "two", "three", "and", "four"};

      KeyStroke keyStroke;
      try {
        keyStroke = KeyStroke.getInstance(contentAssistTrigger);
        IContentProposalProvider contentProposalProvider =
            new SimpleContentProposalProvider(proposals);
        editor.enableContentProposal(new TextContentAdapter(), contentProposalProvider, keyStroke,
            autoActivationCharacters);

      } catch (ParseException ex) {
        // Ignore
      }

      configRegistry.registerConfigAttribute(
          EditConfigAttributes.CELL_EDITABLE_RULE,
          IEditableRule.ALWAYS_EDITABLE);

      configRegistry.registerConfigAttribute(
          EditConfigAttributes.CELL_EDITOR,
          editor,
          DisplayMode.NORMAL,
          COLUMN_ONE_LABEL);
    }
  }
}
